import glob
import os
from pathlib import Path

import pdfplumber
from flask import Flask, render_template, request, redirect, send_file, jsonify
from werkzeug.utils import secure_filename

from search_index import TitleFocusSearchIndex
from slide_indexer import BasicIndexer

app = Flask(__name__)

SLIDE_DIR = "slides"
IMAGE_DIR = "img_cache"
INDEX_DIR = "index"

Index = TitleFocusSearchIndex


@app.route('/')
def index():
    ll = os.listdir(SLIDE_DIR)
    ll.sort()
    return render_template('index.html', ll=ll)


def allowed_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in ["pdf"]


@app.route('/upload', methods=['POST'])
def upload():
    if 'files' in request.files:
        files = request.files.getlist('files')
        index = Index(index_dir=INDEX_DIR)
        indexer = BasicIndexer(index, IMAGE_DIR, SLIDE_DIR)
        for i, file in enumerate(files):
            if file.filename != '':
                if file and allowed_file(file.filename):
                    filename = secure_filename(file.filename)
                    file_path = os.path.join(Path(SLIDE_DIR), filename)
                    file.save(file_path)
                    indexer.add_pdf(file_path)
        del indexer
        del index
    return redirect('/')


@app.route("/search", methods=['POST'])
def query():
    try:
        index = Index(index_dir=INDEX_DIR)
        query = request.form.get("term")
        context = request.form.get("context")
        result = index.search(query, context)
        img_name = result["path"].split(os.sep)[-1] + "_" + str(result["page"]) + ".jpg"
        return jsonify({
            "type": "image",
            "path": os.getenv('EXTERNAL_HOST', '<PLEASE_SET_EXTERNAL_HOST_ENV_VAR>') + "/slide/" + img_name
        })
    except IndexError:
        return jsonify({
            "type": "miss"
        })


@app.route("/reset_index", methods=['POST'])
def reset_index():
    for folder in [IMAGE_DIR, SLIDE_DIR, INDEX_DIR]:
        files = glob.glob(folder + '/*')
        for f in files:
            os.remove(f)
    return redirect('/')


@app.route("/slide/<img_name>")
def slide(img_name):
    path = os.path.join(IMAGE_DIR, img_name)
    if "/" in img_name or "\\" in img_name:
        return "bad image name", 400
    if not os.path.exists(path):
        return "image not present", 404
    return send_file(path, mimetype='image/jpg')


if __name__ == '__main__':
    app.debug = True
    app.run(host="0.0.0.0")
